@@ -205,20 +205,32 @@ class RecordWrapper(object):
             **repr_d)
         return rep
 
+    def __eq__(self, other):
+        """Does not take child order into account!!!!"""
+        if (sorted([c.record['tag'] for c in self]) !=
+                sorted([c.record['tag'] for c in other])):
+            return False
+        if self.__class__.__name__ != other.__class__.__name__:
+            return False
+        for attrib in ['metaname', 'namespace', 'metavalue', 'metadata']:
+            if getattr(self, attrib) != getattr(other, attrib):
+                return False
+        return True
+
     def pop(self, index):
         return self.record['children'].pop(index)
 
     def merge(self, other):
         pass
 
-    def getMemberV(self, tag, container='text'):
+    def getMemberV(self, tag, container='text', default=None, asType=str):
         """Generic accessor for the contents of the children of this element,
         without having to interface with them directly"""
         try:
-            return self.record['children'][self.index(str(tag))][
-                str(container)]
-        except KeyError:
-            return None
+            return asType(self.record['children'][self.index(str(tag))][
+                str(container)])
+        except (KeyError, ValueError):
+            return default
 
     def setMemberV(self, tag, value, container='text'):
         """Generic accessor for the contents of the children of this element,
@@ -420,8 +432,8 @@ class Filters(RecordWrapper):
         if len(self) == 0:
             return True
         return all([sFilt == oFilt for sFilt, oFilt in zip(
-            sorted(list(self)),
-            sorted(list(other)))])
+            sorted(list(self), key=lambda x: x.metaname),
+            sorted(list(other), key=lambda x: x.metaname))])
 
     def __nonzero__(self):
         for filt in self:
@@ -557,6 +569,8 @@ class Filters(RecordWrapper):
     def _bamTypeMap(self):
         return {'rname': str,
                 'length': int,
+                'qstart': int,
+                'qend': int,
                 'qname': str,
                 'movie': str,
                 'zm': int,
@@ -622,6 +636,7 @@ class Filters(RecordWrapper):
                 # renamed:
                 accMap['movie'] = (lambda x: x.MovieID)
                 accMap['qname'] = (lambda x: x.MovieID)
+                accMap['zm'] = (lambda x: x.HoleNumber)
         elif readType == 'fasta':
             accMap = {'id': (lambda x: x.id),
                       'length': (lambda x: x.length.astype(int)),
@@ -654,6 +669,29 @@ class Filters(RecordWrapper):
                         operator = mapOp(req.operator)
                         reqResultsForRecords &= operator(
                             accMap[param](indexRecords), value)
+                    elif param == 'qname':
+                        movie, hole, span = value.split('/')
+                        operator = mapOp(req.operator)
+
+                        value = movieMap[movie]
+                        reqResultsForRecords = operator(
+                            accMap[param](indexRecords), value)
+
+                        param = 'zm'
+                        value = typeMap[param](hole)
+                        reqResultsForRecords &= operator(
+                            accMap[param](indexRecords), value)
+
+                        param = 'qstart'
+                        value = typeMap[param](span.split('_')[0])
+                        reqResultsForRecords &= operator(
+                            accMap[param](indexRecords), value)
+
+                        param = 'qend'
+                        value = typeMap[param](span.split('_')[1])
+                        reqResultsForRecords &= operator(
+                            accMap[param](indexRecords), value)
+
                     else:
                         operator = mapOp(req.operator)
                         reqResultsForRecords = operator(
@@ -977,6 +1015,10 @@ class ExternalResource(RecordWrapper):
             return True
         return False
 
+    @property
+    def uniqueId(self):
+        return self.getV('attrib', 'UniqueId')
+
     def merge(self, other):
         if self.metaType:
             if self.metaType != other.metaType:
@@ -1051,6 +1093,14 @@ class ExternalResource(RecordWrapper):
     def scraps(self, value):
         self._setSubResByMetaType('PacBio.SubreadFile.ScrapsBamFile', value)
 
+    @property
+    def control(self):
+        return self._getSubResByMetaType('PacBio.SubreadFile.Control.SubreadBamFile')
+
+    @control.setter
+    def control(self, value):
+        self._setSubResByMetaType('PacBio.SubreadFile.Control.SubreadBamFile', value)
+
     @property
     def barcodes(self):
         return self._getSubResByMetaType("PacBio.DataSet.BarcodeSet")
@@ -1076,8 +1126,11 @@ class ExternalResource(RecordWrapper):
                 return res.resourceId
 
     def _setSubResByMetaType(self, mType, value):
-        tmp = ExternalResource()
-        tmp.resourceId = value
+        if not isinstance(value, ExternalResource):
+            tmp = ExternalResource()
+            tmp.resourceId = value
+        else:
+            tmp = value
         tmp.metaType = mType
         tmp.timeStampedName = getTimeStampedName(mType)
         resources = self.externalResources
@@ -1122,11 +1175,11 @@ class ExternalResource(RecordWrapper):
             fileIndices = FileIndices(fileIndices[0])
         else:
             fileIndices = FileIndices()
+            self.append(fileIndices)
         for index in list(indices):
             temp = FileIndex()
             temp.resourceId = index
             fileIndices.append(temp)
-        self.append(fileIndices)
 
 class FileIndices(RecordWrapper):
 
@@ -1203,10 +1256,7 @@ class DataSetMetadata(RecordWrapper):
     def numRecords(self):
         """Return the number of records in a DataSet using helper functions
         defined in the base class"""
-        try:
-            return int(self.getMemberV('NumRecords'))
-        except ValueError:
-            return 0
+        return self.getMemberV('NumRecords', default=0, asType=int)
 
     @numRecords.setter
     def numRecords(self, value):
@@ -1218,10 +1268,7 @@ class DataSetMetadata(RecordWrapper):
         """Return the TotalLength property of this dataset.
         TODO: update the value from the actual external reference on
         ValueError"""
-        try:
-            return int(self.getMemberV('TotalLength'))
-        except ValueError:
-            return 0
+        return self.getMemberV('TotalLength', default=0, asType=int)
 
     @totalLength.setter
     def totalLength(self, value):
@@ -1320,10 +1367,7 @@ class ContigSetMetadata(DataSetMetadata):
 
     @property
     def organism(self):
-        try:
-            return self.getMemberV('Organism')
-        except ValueError:
-            return None
+        return self.getMemberV('Organism')
 
     @organism.setter
     def organism(self, value):
@@ -1331,10 +1375,7 @@ class ContigSetMetadata(DataSetMetadata):
 
     @property
     def ploidy(self):
-        try:
-            return self.getMemberV('Ploidy')
-        except ValueError:
-            return None
+        return self.getMemberV('Ploidy')
 
     @ploidy.setter
     def ploidy(self, value):
@@ -1381,10 +1422,7 @@ class BarcodeSetMetadata(DataSetMetadata):
 
     @property
     def barcodeConstruction(self):
-        try:
-            return self.getMemberV('BarcodeConstruction')
-        except ValueError:
-            return None
+        return self.getMemberV('BarcodeConstruction')
 
     @barcodeConstruction.setter
     def barcodeConstruction(self, value):
@@ -1550,32 +1588,35 @@ class StatsMetadata(RecordWrapper):
     """The metadata from the machine sts.xml"""
 
     def merge(self, other):
-        self.shortInsertFraction = (self.shortInsertFraction *
-                                    self.prodDist.bins[1] +
-                                    other.shortInsertFraction *
-                                    other.prodDist.bins[1])/(
-                                        self.prodDist.bins[1]
-                                        + other.prodDist.bins[1])
-        self.adapterDimerFraction = (self.adapterDimerFraction *
-                                     self.prodDist.bins[1] +
-                                     other.adapterDimerFraction *
-                                     other.prodDist.bins[1])/(
-                                         self.prodDist.bins[1]
-                                         + other.prodDist.bins[1])
+        if other.shortInsertFraction and other.prodDist:
+            self.shortInsertFraction = (self.shortInsertFraction *
+                                        self.prodDist.bins[1] +
+                                        other.shortInsertFraction *
+                                        other.prodDist.bins[1])/(
+                                            self.prodDist.bins[1]
+                                            + other.prodDist.bins[1])
+        if other.adapterDimerFraction and other.prodDist:
+            self.adapterDimerFraction = (self.adapterDimerFraction *
+                                         self.prodDist.bins[1] +
+                                         other.adapterDimerFraction *
+                                         other.prodDist.bins[1])/(
+                                             self.prodDist.bins[1]
+                                             + other.prodDist.bins[1])
         self.numSequencingZmws += other.numSequencingZmws
         for dist in DISTLIST:
             selfDist = getattr(self, dist[0].lower() + dist[1:])
             otherDist = getattr(other, dist[0].lower() + dist[1:])
-            try:
-                selfDist.merge(otherDist)
-            except ZeroBinWidthError as e:
-                removed = self.removeChildren(dist)
-                self.append(otherDist)
-            except BinMismatchError:
-                self.append(otherDist)
-            except ValueError:
+            if not selfDist:
                 if otherDist:
                     self.append(otherDist)
+            else:
+                try:
+                    selfDist.merge(otherDist)
+                except ZeroBinWidthError as e:
+                    removed = self.removeChildren(dist)
+                    self.append(otherDist)
+                except BinMismatchError:
+                    self.append(otherDist)
 
     @property
     def prodDist(self):
@@ -1608,11 +1649,21 @@ class StatsMetadata(RecordWrapper):
         return ContinuousDistribution(self.getV('children',
                                                 'InsertReadQualDist'))
 
+    @property
+    def insertReadLenDists(self):
+        return [ContinuousDistribution(child) for child in
+                self.findChildren('InsertReadLenDist')]
+
     @property
     def insertReadLenDist(self):
         return ContinuousDistribution(self.getV('children',
                                                 'InsertReadLenDist'))
     @property
+    def insertReadQualDists(self):
+        return [ContinuousDistribution(child) for child in
+                self.findChildren('InsertReadQualDist')]
+
+    @property
     def controlReadQualDist(self):
         return ContinuousDistribution(self.getV('children',
                                                 'ControlReadQualDist'))
@@ -1633,7 +1684,7 @@ class StatsMetadata(RecordWrapper):
 
     @property
     def adapterDimerFraction(self):
-        return float(self.getMemberV('AdapterDimerFraction'))
+        return self.getMemberV('AdapterDimerFraction', asType=float)
 
     @adapterDimerFraction.setter
     def adapterDimerFraction(self, value):
@@ -1641,7 +1692,7 @@ class StatsMetadata(RecordWrapper):
 
     @property
     def numSequencingZmws(self):
-        return float(self.getMemberV('NumSequencingZmws'))
+        return self.getMemberV('NumSequencingZmws', asType=float)
 
     @numSequencingZmws.setter
     def numSequencingZmws(self, value):
@@ -1649,7 +1700,7 @@ class StatsMetadata(RecordWrapper):
 
     @property
     def shortInsertFraction(self):
-        return float(self.getMemberV('ShortInsertFraction'))
+        return self.getMemberV('ShortInsertFraction', asType=float)
 
     @shortInsertFraction.setter
     def shortInsertFraction(self, value):
@@ -1697,7 +1748,7 @@ class ContinuousDistribution(RecordWrapper):
 
     @property
     def numBins(self):
-        return int(self.getMemberV('NumBins'))
+        return self.getMemberV('NumBins', asType=int)
 
     @numBins.setter
     def numBins(self, value):
@@ -1705,27 +1756,27 @@ class ContinuousDistribution(RecordWrapper):
 
     @property
     def sampleSize(self):
-        return int(self.getMemberV('SampleSize'))
+        return self.getMemberV('SampleSize', asType=int)
 
     @property
     def sampleMean(self):
-        return float(self.getMemberV('SampleMean'))
+        return self.getMemberV('SampleMean', asType=float)
 
     @property
     def sampleMed(self):
-        return float(self.getMemberV('SampleMed'))
+        return self.getMemberV('SampleMed', asType=float)
 
     @property
     def sampleStd(self):
-        return float(self.getMemberV('SampleStd'))
+        return self.getMemberV('SampleStd', asType=float)
 
     @property
     def sample95thPct(self):
-        return float(self.getMemberV('Sample95thPct'))
+        return self.getMemberV('Sample95thPct', asType=float)
 
     @property
     def binWidth(self):
-        return float(self.getMemberV('BinWidth'))
+        return self.getMemberV('BinWidth', asType=float)
 
     @binWidth.setter
     def binWidth(self, value):
@@ -1733,15 +1784,15 @@ class ContinuousDistribution(RecordWrapper):
 
     @property
     def minOutlierValue(self):
-        return float(self.getMemberV('MinOutlierValue'))
+        return self.getMemberV('MinOutlierValue', asType=float)
 
     @property
     def maxOutlierValue(self):
-        return float(self.getMemberV('MaxOutlierValue'))
+        return self.getMemberV('MaxOutlierValue', asType=float)
 
     @property
     def minBinValue(self):
-        return float(self.getMemberV('MinBinValue'))
+        return self.getMemberV('MinBinValue', asType=float)
 
     @minBinValue.setter
     def minBinValue(self, value):
@@ -1749,7 +1800,7 @@ class ContinuousDistribution(RecordWrapper):
 
     @property
     def maxBinValue(self):
-        return float(self.getMemberV('MaxBinValue'))
+        return self.getMemberV('MaxBinValue', asType=float)
 
     @maxBinValue.setter
     def maxBinValue(self, value):
@@ -1838,7 +1889,7 @@ class DiscreteDistribution(RecordWrapper):
 
     @property
     def numBins(self):
-        return int(self.getMemberV('NumBins'))
+        return self.getMemberV('NumBins', asType=int)
 
     @property
     def bins(self):
@@ -1876,6 +1927,10 @@ class RunDetailsMetadata(RecordWrapper):
     def name(self):
         return self.getMemberV('Name')
 
+    @name.setter
+    def name(self, value):
+        return self.setMemberV('Name', value)
+
 
 class WellSampleMetadata(RecordWrapper):
 
